/* Copyright (C) 2005 Chris Vine

This program is distributed under the General Public Licence, version 2.
For particulars of this and relevant disclaimers see the file
COPYING distributed with the source files.

*/

#ifndef MUTEX_H
#define MUTEX_H

#include "prog_defs.h"

#include <glib/gthread.h>

namespace Thread {

class Cond;

class Mutex {
  GMutex* g_mutex_p;

  // mutexes cannot be copied
  Mutex(const Mutex&);
  Mutex& operator=(const Mutex&);
public:
  class Lock;
  friend class Cond;

  void lock() {g_mutex_lock(g_mutex_p);}
  bool trylock() {return g_mutex_trylock(g_mutex_p);}
  void unlock() {g_mutex_unlock(g_mutex_p);}

  Mutex(void): g_mutex_p(g_mutex_new()) {}
  ~Mutex(void) {g_mutex_free(g_mutex_p);}
};

class Mutex::Lock {
  Mutex& mutex;

  // locks cannot be copied
  Lock(const Mutex::Lock&);
  Mutex::Lock& operator=(const Mutex::Lock&);
public:

  Lock(Mutex& mutex_): mutex(mutex_) {mutex.lock();}
  ~Lock() {mutex.unlock();}
};

class Cond {
  GCond* g_cond_p;

  // Cond cannot be copied
  Cond(const Cond&);
  Cond& operator=(const Cond&);
public:
  void signal() {g_cond_signal(g_cond_p);}
  void broadcast() {g_cond_broadcast(g_cond_p);}
  void wait(Mutex& mutex) {g_cond_wait(g_cond_p, mutex.g_mutex_p);}

  Cond(): g_cond_p(g_cond_new()) {}
  ~Cond() {g_cond_free(g_cond_p);}
};


} // namespace Thread

#endif
