/* Copyright (C) 2005 Chris Vine

This program is distributed under the General Public Licence, version 2.
For particulars of this and relevant disclaimers see the file
COPYING distributed with the source files.

*/


#include "thread.h"
#include "sem_sync.h"

struct ThreadArgs {
  const sigc::slot<void>* slot_p;
  SemSync* sem_p;
};

namespace Thread {

void* thread_func(void* arg) {

  // this is executing in the new thread
  ThreadArgs* thread_args_p = static_cast<ThreadArgs*>(arg);
  sigc::slot<void> callback = *(thread_args_p->slot_p);
  thread_args_p->sem_p->post();
  // at this point the pointers in pthread_args will cease to be valid
  // but that's OK as we now hold our own local copy of the slot and
  // we have finished with the semaphore
  callback();
  return 0;
}

SharedPtr<Thread> Thread::start(const sigc::slot<void>& cb, bool joinable) {

  Thread* instance_p = new Thread;

  ThreadArgs thread_args;
  SemSync sem;
  thread_args.slot_p = &cb;
  thread_args.sem_p = &sem;

  GThread* thread_p = g_thread_create(thread_func, &thread_args, joinable, 0);
  if (thread_p) {
    instance_p->g_thread_p = thread_p;
    sem.wait();
    return SharedPtr<Thread>(instance_p);
  }
  else {
    delete instance_p;
    return SharedPtr<Thread>(0);
  }
}

} // namespace Thread
